/*************************************************************************
**************************************************************************
**                                                                      **
**  Project:   PS/2 Keyboard => NKC                                     **
**  Title:     PS/2 Interface => NKC Implementation                     **
**  File:      USART.c                                                  **
**  Date:      29.11.2008                                               **
**  Version:   0.82                                                     **
**  Plattform: WinAVR-20060421 & AVR Studio 4.12.498 & STK500           **
**                                                                      **
**  Created by Gerald Ebert                                             **
**                                                                      **
**************************************************************************
 Version   Description/Changes

 Version   Description/Changes

   0.80    Project start
   0.82    Changes to support different cpu models

**************************************************************************
*************************************************************************/

#include "avr_defs.h"
#include "usart.h"

//========================================================================
// CPU model dependant defines
//========================================================================

#if defined(__AVR_ATmega48__) || defined(__AVR_ATmega88__) || defined(__AVR_ATmega168__)

  #define  USART_SETBAUDRATE(BaudrateValue) { UBRR0H = (BYTE)(BaudrateValue>>8);  UBRR0L = (BYTE)(BaudrateValue&0xFF); }
  #define  USART_ENABLE_RXTX(ParityMode)    { UCSR0B = _BV(RXEN0)|_BV(TXEN0);  UCSR0C = (ParityMode<<UPM00)|(3<<UCSZ00); }
  #define  USART_ENABLE_TX(ParityMode)      { UCSR0B = _BV(TXEN0);  UCSR0C = (ParityMode<<UPM00)|(3<<UCSZ00); }
  #define  USART_WAIT_TXEMPTY               { while (! (UCSR0A & _BV(UDRE0)) ); }
  #define  USART_TXDATA(Data)               { UDR0 = Data; }
  #define  USART_WAIT_RXCOMPLETE            { while (! (UCSR0A & BV(RXC0)) );
  #define  USART_RXDATA(Data)               { Data = UDR0; }
  #define  USART_GETSTATUS(Status)          { Status = UCSR0A; }

#elif defined(__AVR_ATtiny2313__)

  #define  USART_SETBAUDRATE(BaudrateValue) { UBRRH = (BYTE)(BaudrateValue>>8);  UBRRL = (BYTE)(BaudrateValue&0xFF); }
  #define  USART_ENABLE_RXTX(ParityMode)    { UCSRB = _BV(RXEN)|_BV(TXEN);  UCSRC = (ParityMode<<UPM0)|(3<<UCSZ0); }
  #define  USART_ENABLE_TX(ParityMode)      { UCSRB = _BV(TXEN);  UCSRC = (ParityMode<<UPM0)|(3<<UCSZ0); }
  #define  USART_WAIT_TXEMPTY               { while (! (UCSRA & _BV(UDRE)) ); }
  #define  USART_TXDATA(Data)               { UDR = Data; }
  #define  USART_WAIT_RXCOMPLETE            { while (! (UCSRA & BV(RXC)) ); }
  #define  USART_RXDATA(Data)               { Data = UDR; }
  #define  USART_GETSTATUS(Status)          { Status = UCSRA; }

#else

  #error "CPU model not supported."

#endif

//========================================================================
//========================================================================
// USART Functions
//========================================================================
//========================================================================

//------------------------------------------------------------------------
//  Initialize USART and set baudrate
//------------------------------------------------------------------------

void  USART_Init(WORD BaudrateValue, BYTE ParityMode, BYTE StopBits)
{
   USART_SETBAUDRATE(BaudrateValue);

#if defined(USART_RECEIVE)
   USART_ENABLE_RXTX(ParityMode);
#else
   USART_ENABLE_TX(ParityMode);
#endif
}

//------------------------------------------------------------------------
//  Send Data to client
//------------------------------------------------------------------------

void USART_Transmit(BYTE Data)
{
   // Wait for empty transmit buffer
   USART_WAIT_TXEMPTY;
   // Put data into buffer to start sending the data
   USART_TXDATA(Data);
}

//------------------------------------------------------------------------
//  Send a string
//------------------------------------------------------------------------

void USART_TransmitString(LPCSTR str)
{
   while ( *str )
	   USART_Transmit((BYTE)*(str++));
}

//------------------------------------------------------------------------
//  Send a string from program memory
//------------------------------------------------------------------------

void USART_TransmitPString(LPCPSTR str)
{
   char ch;

   while ( (ch = GET_PCHAR(str++)) )
	  USART_Transmit((BYTE)ch);
}

//------------------------------------------------------------------------
//  Send buffer content
//------------------------------------------------------------------------

void USART_TransmitBuffer(LPCBYTE buffer, INT buflen)
{
   while ( buflen-- )
      USART_Transmit(*(buffer++));
}

//------------------------------------------------------------------------
//  Receive Data
//------------------------------------------------------------------------
#if defined(USART_RECEIVE)

BYTE USART_Receive(void)
{
   BYTE   Data;
   
   // Wait for data to be received
   USART_WAIT_RXCOMPLETE;
   // Return received data
   USART_RXDATA(Data);
   
   return Data;
}

#endif

//========================================================================
