/*************************************************************************
**************************************************************************
**                                                                      **
**  Project:   common                                                   **
**  Title:     Usefull functions                                        **
**  File:      avr_defs.c                                               **
**  Date:      29.11.2008                                               **
**  Version:   1.03                                                     **
**  Plattform: WinAVR-20060421 & AVR Studio 4.12.498 & STK500           **
**                                                                      **
**  Created by Gerald Ebert                                             **
**                                                                      **
**************************************************************************
*************************************************************************/

#include "avr_defs.h"

//========================================================================
//========================================================================
// Delay Functions
//========================================================================
//========================================================================

void  Wait_us(WORD us)
{
   WORD count;

#if F_CPU <= 4000000
   BYTE i = 1;
#elif F_CPU <= 8000000
   BYTE i = 2;
#elif F_CPU <= 12000000
   BYTE i = 3;
#else 
   BYTE i = 4;
#endif

   while ( i-- )
		// 16-bit count, 4 cycles/loop
		__asm__ __volatile__ ( "movw %0, %1"  "\n\t" "1: sbiw %0,1" "\n\t" "brne 1b" "\n\t" : "=&w"(count) : "w"(us) );
}

//========================================================================
//========================================================================
// WATCHDOG TIMER
//========================================================================
//========================================================================

//------------------------------------------------------------------------
// Enable Watchdog Timer Interrupt Routine 
//------------------------------------------------------------------------

void WDTI_IntEnable(BYTE Timeout)
{
#if defined(__AVR_ATmega48__) || defined(__AVR_ATmega88__) || defined(__AVR_ATmega168__) || \
    defined(__AVR_ATtiny2313__)

   CLI;                                          // Block all used ISR
   WDT;                                          // Reset Watchdog Timer 
   WDTCSR |= BV(WDCE) | BV(WDE);                 // Prepare control flag configuration
   WDTCSR = ((Timeout&0x07) | BV(WDIE));         // Set WDT Interrupt and Timer Counter
   SEI;                                          // Enable all used ISR

#else

  #error "CPU model not supported."

#endif
}

//------------------------------------------------------------------------
// Disable Watchdog Timer
//------------------------------------------------------------------------

void WDTI_Disable()
{
#if defined(__AVR_ATmega48__) || defined(__AVR_ATmega88__) || defined(__AVR_ATmega168__) || \
    defined(__AVR_ATtiny2313__)

   CLI;                                          // Block all used ISR
   WDT;                                          // Reset Watchdog Timer 
   MCUSR &= ~BV(WDRF);                           // Clear Reset flag
   WDTCSR |= BV(WDCE) | BV(WDE);                 // Prepare control flag configuration
   WDTCSR = 0;                                   // Disable Watchdog Timer
   SEI;                                          // Enable all used ISR

#else

  #error "CPU model not supported."

#endif
}

//========================================================================
